﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using System.Drawing;

namespace Dscf.CustomerCenter.Utils
{
    public class FileUtil : System.Web.UI.Page
    {
        /// <summary>
        /// 文件上传
        /// </summary>
        /// <param name="fileMessage">文件生成方式</param>
        /// <returns></returns>
        public UpFileResultMessage UpLoadFile(UpFileMessage fileMessage)
        {
            try
            {
                string now = DateTime.Today.ToString("yyyyMMdd");
                string guid = Guid.NewGuid().ToString();

                //获取文件扩展名
                var fileSuffix = Path.GetExtension(fileMessage.OriginalFileName);

                var uploadFolder = Path.Combine(System.Web.HttpContext.Current.Server.MapPath(ParmsConfig.UpFilePathUrl), now);

                if (!Directory.Exists(uploadFolder))
                {
                    Directory.CreateDirectory(uploadFolder);
                }

                //保存文件名
                string saveFileName = guid + fileSuffix;
                string filePath = Path.Combine(uploadFolder, saveFileName);


                UpFileResultMessage upFileResult = new UpFileResultMessage()
                {
                    IsSuccess = true,
                    FileName = Path.GetFileNameWithoutExtension(fileMessage.OriginalFileName),
                    FileSuffix = fileSuffix,
                    FilePath = string.Format(@"{0}/{1}", ParmsConfig.UpFileIPAddressUrl, now),
                    SaveFileName = guid
                };

                using (Stream sourceStream = fileMessage.FileStream)
                {
                    using (FileStream targetStream = new FileStream(filePath, FileMode.Create, FileAccess.Write, FileShare.None))
                    {
                        const int bufferLen = 1024 * 4;//4KB
                        byte[] buffer = new byte[bufferLen];
                        int count = 0;
                        while ((count = sourceStream.Read(buffer, 0, bufferLen)) > 0)
                        {
                            targetStream.Write(buffer, 0, count);
                        }
                    }
                    //上传文件为图片时,需创建缩略图
                    if (fileMessage.IsImage)
                    {
                        var uploadThumbFolder = Path.Combine(uploadFolder, "Thumb");

                        if (!Directory.Exists(uploadThumbFolder))
                        {
                            Directory.CreateDirectory(uploadThumbFolder);
                        }
                        using (FileStream targetStream = new FileStream(filePath, FileMode.Open, FileAccess.Read, FileShare.None))
                        {
                            System.Drawing.Image image = System.Drawing.Image.FromStream(targetStream);
                            int width = image.Width;
                            int height = image.Height;
                            int thumbWidth = 0;
                            int thumbHeight = 0;
                            switch (fileMessage.Mode)
                            {
                                case "WH":  //指定高宽缩放（可能变形）  
                                    thumbWidth = fileMessage.ThumbWidth.HasValue ? fileMessage.ThumbWidth.Value : 200;
                                    thumbHeight = fileMessage.ThumbHeight.HasValue ? fileMessage.ThumbHeight.Value : 200;
                                    break;
                                case "W":   //指定宽，高按比例     
                                    thumbWidth = fileMessage.ThumbWidth.HasValue ? fileMessage.ThumbWidth.Value : 200;
                                    thumbHeight = height * thumbWidth / width;
                                    break;
                                case "H":   //指定高，宽按比例
                                    thumbHeight = fileMessage.ThumbHeight.HasValue ? fileMessage.ThumbHeight.Value : 200;
                                    thumbWidth = width * thumbHeight / height;
                                    break;
                                default:
                                    thumbWidth = fileMessage.ThumbWidth.HasValue ? fileMessage.ThumbWidth.Value : 200;
                                    thumbHeight = height * thumbWidth / width;
                                    break;
                            }
                            string thumbFilePath = Path.Combine(uploadThumbFolder, saveFileName);
                            CreateThumbnail(thumbFilePath, targetStream, thumbWidth, thumbHeight);
                            upFileResult.ThumbPath = string.Format(@"{0}/{1}/Thumb", ParmsConfig.UpFileIPAddressUrl, now);
                        }
                    }
                }

                return upFileResult;
            }
            catch (Exception ex)
            {
                return new UpFileResultMessage() { IsSuccess = false, Message = ex.Message };
            }

        }

        /// <summary>
        /// 创建指定图片文件流的缩略图
        /// </summary>
        /// <param name="thumbFilePath">缩略图文件保存路径</param>
        /// <param name="fileStream">原始文件流</param>
        /// <param name="width">缩略图宽</param>
        /// <param name="height">缩略图高</param>
        private void CreateThumbnail(string thumbFilePath, Stream fileStream, int width, int height)
        {
            using (Image image = Image.FromStream(fileStream))
            {
                using (Image thumbnail = image.GetThumbnailImage(width, height, null, IntPtr.Zero))
                {
                    thumbnail.Save(thumbFilePath);
                }
            }

        }

    }
}
